---
layout: default
title: OWNER &bull; Java&trade; properties reinvented.
overview: true
---

<section class="intro">
    <div class="grid">
        <div class="unit whole center-on-mobiles">
            <p class="first">{{ site.description }}</p>
        </div>
    </div>
</section>

<section class="features">
    <div class="grid">
        <div class="unit one-third">
            <h2>Simple</h2>
            <p>
                No more code to load, convert and manage properties files, just use <em>your properties</em> in the simplest possible way.
            </p>
            <a href="{{ site.url }}/docs/usage/" class="">See how OWNER works &rarr;</a>
        </div>
        <div class="unit one-third">
            <h2>Powerful</h2>
            <p>
                Annotation based, automatic type conversion, variables expansion, load strategies, hot reload, event notifications, and <em>much</em> more...
            </p>
            <a href="{{ site.url }}/docs/features/" class="">See OWNER features list &rarr;</a>
        </div>
        <div class="unit one-third">
            <h2>Flexible</h2>
            <p>
                Pick the features you need and forget what you don't need. Easy to adopt, scales on complexity, fully documented.
            </p>
            <a href="{{ site.url }}/docs/why/" class="">Why should I use OWNER? &rarr;</a>
        </div>
        <div class="clear"></div>
    </div>
</section>

<section class="quickstart">
    <div class="grid">
        <div class="unit golden-small center-on-mobiles">
            <h4>See how it works!</h4>
        </div>
        <div class="unit golden-large code" style="width: 550px;">
            <p class="title">OWNER Overview</p>
            <div class="shell" style="text-align: center;">
                <iframe src="http://www.slideshare.net/slideshow/embed_code/31716769?rel=0"
                        width="500" height="416"
                        style="max-width: 100%; padding: 5px 0px 0px 0px;"
                        frameborder="0" marginwidth="0" marginheight="0" scrolling="no"
                        allowfullscreen> </iframe>
            </div>
        </div>
        <div class="clear"></div>
    </div>
</section>

