---
title: OWNER • Java™ properties reinvented.
---
<html>
    <head>
        <meta http-equiv="refresh" content="0; url={{ site.url }}/docs/welcome/">
    </head>
    <body>
        <p>
            If your browser supports the refresh meta tag, you'll be redirected 
            to the <a href="{{ site.url }}/docs/welcome/">welcome</a> page 
            automatically. Otherwise, select the link manually.
        </p>
    </body>
</html>
