<div class="section-nav">
  <div class="left align-right">
    {% if page.prev_section != null %}
      <a href="{{ site.url }}/docs/{{ page.prev_section }}" class="prev">
        Back
      </a>
    {% else %}
      <span class="prev disabled">Back</span>
    {% endif %}
  </div>
  <div class="right align-left">
    {% if page.next_section != null %}
      <a href="{{ site.url }}/docs/{{ page.next_section }}" class="next">
        Next
      </a>
    {% else %}
      <span class="next disabled">Next</span>
    {% endif %}
  </div>
  <div class="clear"></div>
</div>

