<div class="docs-nav-mobile unit whole show-on-mobiles">
    <select onchange="if (this.value) window.location.href=this.value">

        <option value="">Navigate the docs…</option>

        <optgroup label="Getting started">
            <option value="{{ site.url }}/docs/welcome/">Welcome</option>
            <option value="{{ site.url }}/docs/installation/">Installation
            </option>
            <option value="{{ site.url }}/docs/usage/">Basic usage</option>
        </optgroup>

        <optgroup label="More Features">
            <option value="{{ site.url }}/docs/loading-strategies/">Loading strategies</option>
            <option value="{{ site.url }}/docs/importing-properties/">Importing properties</option>
            <option value="{{ site.url }}/docs/parametrized-properties/">Parametrized properties</option>
            <option value="{{ site.url }}/docs/type-conversion/">Type conversion</option>
            <option value="{{ site.url }}/docs/variables-expansion/">Variables expansion</option>
            <option value="{{ site.url }}/docs/reload/">Reload and Hot Reload</option>
            <option value="{{ site.url }}/docs/accessible-mutable/">Accessible and Mutable</option>
            <option value="{{ site.url }}/docs/debugging/">Debugging</option>
            <option value="{{ site.url }}/docs/disabling-features/">Disabling features</option>
            <option value="{{ site.url }}/docs/configuring/">Metaconfiguring</option>
            <option value="{{ site.url }}/docs/xml-support/">XML support</option>
            <option value="{{ site.url }}/docs/event-support/">Event support</option>
            <option value="{{ site.url }}/docs/singleton/">Singleton</option>
            <option value="{{ site.url }}/docs/crypto/">Crypto Support</option>
        </optgroup>

        <optgroup label="Miscellaneous">
            <option value="{{ site.url }}/docs/features/">Features</option>
            <option value="{{ site.url }}/docs/why/">Why OWNER?</option>
            <option value="{{ site.url }}/docs/faq/">FAQ</option>
            <option value="{{ site.url }}/docs/support/">Getting support</option>
        </optgroup>

        <optgroup label="Meta">
            <option value="{{ site.url }}/docs/building/">Building from sources</option>
            <option value="{{ site.url }}/docs/contributing/">Contributing</option>
            <option value="{{ site.url }}/docs/license/">License</option>
        </optgroup>

        <optgroup label="Other stuff">
            <option value="{{ site.url }}/news/releases/">History</option>
            <option value="{{ site.url }}/apidocs/latest/">Javadoc</option>
            <option value="http://search.maven.org/#search%7Cgav%7C1%7Cg%3A%22org.aeonbits.owner%22%20AND%20a%3A%22owner%22">Downloads</option>
            <option value="https://github.com/lviggiano/owner/releases">Releases</option>
            <option value="https://t.me/ownerapi">Telegram Chat</option>
            <option value="https://travis-ci.org/lviggiano/owner">Travis CI reports</option>
            <option value="https://sonarcloud.io/dashboard?id=org.aeonbits.owner%3Aowner-parent">Sonar reports</option>
            <option value="https://coveralls.io/r/lviggiano/owner">Coveralls reports</option>
            <option value="https://www.ohloh.net/p/owner">OWNER at Ohloh</option>
        </optgroup>

    </select>
</div>
