/*
 * Copyright (c) 2012-2015, Luigi R. Viggiano
 * All rights reserved.
 *
 * This software is distributable under the BSD license.
 * See the terms of the BSD license in the documentation provided with this software.
 */

/**
 * Provides interfaces and classes for dealing with different types of events fired by OWNER components.
 *
 * See the {@link org.aeonbits.owner.event.Event} class. Events are fired by event sources.
 * An event listener registers with an event source to receive notifications about the events of a
 * particular type. This package defines events and event listeners, as well as event listener adapters,
 * which are convenience classes to make easier the process of writing event listeners.
 */
package org.aeonbits.owner.event;
