<ul>
    <li class="{% if page.title == "Loading strategies" %}current{% endif %}">
    <a href="{{ site.url }}/docs/loading-strategies">Loading strategies</a>
    </li>
    <li class="{% if page.title == "Importing properties" %}current{% endif %}">
    <a href="{{ site.url }}/docs/importing-properties">Importing properties</a>
    </li>
    <li class="{% if page.title == "Parametrized properties" %}current{% endif %}">
    <a href="{{ site.url }}/docs/parametrized-properties/">Parametrized properties</a>
    </li>
    <li class="{% if page.title == "Type conversion" %}current{% endif %}">
    <a href="{{ site.url }}/docs/type-conversion/">Type conversion</a>
    </li>
    <li class="{% if page.title == "Variables expansion" %}current{% endif %}">
    <a href="{{ site.url }}/docs/variables-expansion/">Variables expansion</a>
    </li>
    <li class="{% if page.title == "Reload and Hot Reload" %}current{% endif %}">
    <a href="{{ site.url }}/docs/reload/">Reload and Hot Reload</a>
    </li>
    <li class="{% if page.title == "Accessible and Mutable" %}current{% endif %}">
    <a href="{{ site.url }}/docs/accessible-mutable/">Accessible and Mutable</a>
    </li>
    <li class="{% if page.title == "Debugging" %}current{% endif %}">
    <a href="{{ site.url }}/docs/debugging/">Debugging</a>
    </li>
    <li class="{% if page.title == "Disabling features" %}current{% endif %}">
    <a href="{{ site.url }}/docs/disabling-features/">Disabling features</a>
    </li>
    <li class="{% if page.title == "Configuring the ConfigFactory" %}current{% endif %}">
    <a href="{{ site.url }}/docs/configuring/">Metaconfiguring</a>
    </li>
    <li class="{% if page.title == "XML support" %}current{% endif %}">
    <a href="{{ site.url }}/docs/xml-support/">XML support</a>
    </li>
    <li class="{% if page.title == "Event support" %}current{% endif %}">
    <a href="{{ site.url }}/docs/event-support/">Event support</a>
    </li>
    <li class="{% if page.title == "Singleton" %}current{% endif %}">
    <a href="{{ site.url }}/docs/singleton/">Singleton</a>
    </li>
    <li class="{% if page.title == "Crypto support" %}current{% endif %}">
    <a href="{{ site.url }}/docs/crypto/">Crypto support</a>
    </li>
</ul>
