/*
 * Copyright (c) 2012-2015, Luigi R. Viggiano
 * All rights reserved.
 *
 * This software is distributable under the BSD license.
 * See the terms of the BSD license in the documentation provided with this software.
 */

/**
 * Provides interfaces and classes to allow OWNER to load properties from several file formats.
 */
package org.aeonbits.owner.loaders;
