<ul>
  <li class="{% if page.overview %}current{% endif %}">
    <a href="{{ site.url }}/">Overview</a>
  </li>
  <li class="{% if page.url contains '/docs/' %}current{% endif %}">
    <a href="{{ site.url }}/docs/welcome">Doc<span class="show-on-mobiles">s</span><span class="hide-on-mobiles">umentation</span></a>
  </li>
  <li class="{% if page.author %}current{% endif %}">
    <a href="{{ site.url }}/news">News</a>
  </li>
  <li class="">
    <a href="https://github.com/{{ site.author.github }}/{{ site.project.github }}"><span class="hide-on-mobiles">View on </span>GitHub</a>
  </li>
</ul>
