<footer>
  <div class="grid">
    <div class="unit one-third center-on-mobiles">
      <p>Project maintained by <a href="https://github.com/{{ site.author.github }}">{{ site.author.name}}</a>,
          and the <a href="https://github.com/{{ site.author.github }}/{{ site.project.github }}/graphs/contributors">awesome contributors</a>.</p>
    </div>
    <div class="unit one-third align-right center-on-mobiles">
      <p>Developed with
        <a href="http://www.jetbrains.com/idea">
          <img style="width: 30px; height: 30px" src="{{ site.url }}/img/ij-logo.png" alt="IntelliJ IDEA">
        </a>
      </p>
    </div>
    <div class="unit one-third align-right center-on-mobiles">
      <p>
        Hosted on
        <a href="https://github.com">
          <img src="{{ site.url }}/img/footer-logo.png" alt="GitHub • Social coding">
        </a>
      </p>
    </div>
  </div>
</footer>