---
layout: default
---

  <section class="docs">
    <div class="grid">

      {% include docs_contents_mobile.html %}

      <div class="unit four-fifths">
        <article>
          <h1>{{ page.title }}</h1>
          {{ content }}
          {% include section_nav.html %}
        </article>
      </div>

      {% include docs_contents.html %}

      <div class="clear"></div>

    </div>
  </section>
