<div class="unit one-fifth hide-on-mobiles">
  <aside>

    <h4>Getting Started</h4>
    <ul>
      <li class="{% if page.title == "Welcome" %}current{% endif %}">
        <a href="{{ site.url }}/docs/welcome">Welcome</a>
      </li>
      <li class="{% if page.title == "Installation" %}current{% endif %}">
        <a href="{{ site.url }}/docs/installation">Installation</a>
      </li>
      <li class="{% if page.title == "Basic usage" %}current{% endif %}">
        <a href="{{ site.url }}/docs/usage">Basic usage</a>
      </li>
    </ul>

    <h4>More Features</h4>
    {% include features.html %}

    <h4>Miscellaneous</h4>
    <ul>
        <li class="{% if page.title == "Features" %}current{% endif %}">
            <a href="{{ site.url }}/docs/features/">Features</a>
        </li>
        <li class="{% if page.title == "Why should I use OWNER?" %}current{% endif %}">
          <a href="{{ site.url }}/docs/why/">Why OWNER?</a>
        </li>
        <li class="{% if page.title == "Frequently Asked Questions" %}current{% endif %}">
            <a href="{{ site.url }}/docs/faq/">FAQ</a>
        </li>

        <li class="{% if page.title == "Getting support" %}current{% endif %}">
            <a href="{{ site.url }}/docs/support/">Getting support</a>
        </li>

    </ul>

    <h4>Meta</h4>
    <ul>
      <li class="{% if page.title == "Building from sources" %}current{% endif %}">
        <a href="{{ site.url }}/docs/building">Building from sources</a>
      </li>
      <li class="{% if page.title == "Contributing" %}current{% endif %}">
        <a href="{{ site.url }}/docs/contributing">Contributing</a>
      </li>
      <li class="{% if page.title == "License" %}current{% endif %}">
        <a href="{{ site.url }}/docs/license">License</a>
      </li>
    </ul>

    <h4>Other stuff</h4>
    <ul>
      <li><a href="{{ site.url }}/news/releases/">History</a></li>
      <li><a href="{{ site.url }}/apidocs/latest/index.html">Javadoc</a></li>
      <li><a href="http://search.maven.org/#search%7Cgav%7C1%7Cg%3A%22org.aeonbits.owner%22%20AND%20a%3A%22owner%22">Downloads</a></li>
      <li><a href="https://github.com/lviggiano/owner/releases">Releases</a></li>
      <li><a href="https://t.me/ownerapi">Telegram Chat</a></li>
      <li><a href="https://travis-ci.org/lviggiano/owner">Travis CI reports</a></li>
      <li><a href="https://sonarcloud.io/dashboard?id=org.aeonbits.owner%3Aowner-parent">Sonar reports</a></li>
      <li><a href="https://coveralls.io/r/lviggiano/owner">Coveralls reports</a></li>
      <li><a href="https://www.ohloh.net/p/owner">OWNER at Ohloh</a></li>
    </ul>

  </aside>
</div>
