<header>
  <nav class="mobile-nav show-on-mobiles">
    {% include primary-nav-items.html %}
  </nav>
  <div class="grid">
    <div class="unit one-third center-on-mobiles">
      <a href="{{ site.url }}/">
        <h1>
          <span class="shadow">{{ site.project.name }}</span>
        </h1>
        <small class="shadow">Java&trade; properties reinvented.</small>
      </a>
    </div>
    <nav class="main-nav unit two-thirds hide-on-mobiles">
      {% include primary-nav-items.html %}
    </nav>
  </div>
  <div class="ribbon">
    <a href="https://github.com/lviggiano/owner"><img style="position: absolute; top: 0; right: 0; border: 0;" src="https://camo.githubusercontent.com/652c5b9acfaddf3a9c326fa6bde407b87f7be0f4/68747470733a2f2f73332e616d617a6f6e6177732e636f6d2f6769746875622f726962626f6e732f666f726b6d655f72696768745f6f72616e67655f6666373630302e706e67" alt="Fork me on GitHub" data-canonical-src="https://s3.amazonaws.com/github/ribbons/forkme_right_orange_ff7600.png"></a>
  </div>
</header>
