/*
 * Copyright (c) 2012-2015, Luigi R. Viggiano
 * All rights reserved.
 *
 * This software is distributable under the BSD license.
 * See the terms of the BSD license in the documentation provided with this software.
 */

package org.aeonbits.owner.dummy;

/**
 * Dummy class to work around bugs:
 *
 * <ul>
 *  <li><a href="https://issues.sonatype.org/browse/OSSRH-3723">OSSRH-3723</a></li>
 *  <li><a href="https://issues.sonatype.org/browse/OSSRH-637">OSSRH-637</a></li>
 * </ul>
 *
 * <p>This class is useless; please just ignore it.</p>
 * <p>It is just needed to allow the release on SonaType, since validation is failing for missing javadocs.</p>
 *
 * @author Luigi R. Viggiano
 */
public class DummyJava8Readme {
}
