<!DOCTYPE HTML>
<html lang="en-US">
<head>
  <meta charset="UTF-8">
  <title>{{ page.title }}</title>
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
  <meta name="viewport" content="width=device-width,initial-scale=1">
  <link rel="alternate" type="application/rss+xml" title="{{ site.title }} - Feed" href="/feed.xml" />
  <link rel="alternate" type="application/atom+xml" title="Recent commits to OWNER’s master branch" href="https://github.com/lviggiano/owner/commits/master.atom" />
  <link href='http://fonts.googleapis.com/css?family=Lato:100,300,400,700,900,100italic,300italic,400italic,700italic,900italic' rel='stylesheet' type='text/css'>
  <link href='http://fonts.googleapis.com/css?family=Arizonia' rel='stylesheet' type='text/css'>
  <link rel="stylesheet" href="{{ site.url }}/css/normalize.css" />
  <link rel="stylesheet" href="{{ site.url }}/css/gridism.css" />
  <link rel="stylesheet" href="{{ site.url }}/css/style.css" />
  <link rel="stylesheet" href="{{ site.url }}/css/pygments.css" />
  <link rel="icon" type="image/x-icon" href="{{ site.url }}/favicon.png" />
  <script src="{{ site.url }}/js/modernizr-2.5.3.min.js"></script>
</head>
